==> fraiseql-1.1.4.tar.gz: fraiseql-1.1.4/tests/integration/auth/test_security_validation.py <==
"""Test security validation logic for FraiseQL."""

import pytest


@pytest.mark.security
class TestFunctionNameValidation:
    """Test the function name validation logic that prevents SQL injection."""

    def validate_function_name(self, function_name: str) -> bool:
        """Replicate the validation logic from db.py."""
        return function_name.replace("_", "").replace(".", "").isalnum()

    def test_valid_function_names(self) -> None:
        """Test that valid function names pass validation."""
        valid_names = [
            "create_user",
            "api.create_user",
            "schema123.function_name",
            "simple_function",
            "func123",
            "a",
            "a.b",
            "a_b",
            "func1_test2.schema3",
            "graphql.create_user",
            "public.health_check",
            "user_management.update_profile",
        ]

        for name in valid_names:
            assert self.validate_function_name(name), f"'{name}' should be valid"

    def test_malicious_function_names_rejected(self) -> None:
        """Test that malicious function names are rejected."""
        malicious_names = [
            "'; DROP TABLE users; --",
            "create_user'; DELETE FROM users; --",
            "func() UNION SELECT * FROM passwords",
            "func; INSERT INTO admin VALUES",
            "func'||pg_sleep(10)||'",
            'func"; TRUNCATE TABLE users; --',
            "func\nDROP DATABASE production",
            "func OR 1=1",
            "func) AS (SELECT password FROM users",
            "func/**/UNION/**/SELECT/**/*",
            "func' AND '1'='1",
            "func'; UPDATE users SET admin=true; --",
        ]

        for name in malicious_names:
            assert not self.validate_function_name(name), f"'{name}' should be rejected"

    def test_edge_case_function_names(self) -> None:
        """Test edge cases for function name validation."""
        edge_cases = [
            ("", False),  # Empty string
            ("func-name", False),  # Hyphen not allowed
            ("func name", False),  # Space not allowed
            ("func@domain", False),  # @ not allowed
            ("func$var", False),  # $ not allowed
            ("func#tag", False),  # # not allowed
            ("func%", False),  # % not allowed
            ("func*", False),  # * not allowed
            ("func()", False),  # Parentheses not allowed
            ("func[0]", False),  # Brackets not allowed
            ("func{}", False),  # Braces not allowed
            ("func<>", False),  # Angle brackets not allowed
            ("func|pipe", False),  # Pipe not allowed
            ("func\\backslash", False),  # Backslash not allowed
            ("func/slash", False),  # Forward slash not allowed
            ("func:colon", False),  # Colon not allowed
            ("func;semicolon", False),  # Semicolon not allowed
            ("func'quote", False),  # Single quote not allowed
            ('func"doublequote', False),  # Double quote not allowed
            ("func`backtick", False),  # Backtick not allowed
            ("func+plus", False),  # Plus not allowed
            ("func=equals", False),  # Equals not allowed
            ("func\ttab", False),  # Tab not allowed
            ("func\nnewline", False),  # Newline not allowed
            ("func\rcarriage", False),  # Carriage return not allowed
        ]

        for function_name, should_pass in edge_cases:
            result = self.validate_function_name(function_name)
            if should_pass:
                assert result, f"'{function_name}' should be valid but was rejected"
            else:
                assert not result, f"'{function_name}' should be invalid but was accepted"

    def test_validation_prevents_common_sql_injection_patterns(self) -> None:
        """Test that validation prevents common SQL injection attack patterns."""
        injection_patterns = [
            # Comment-based injections
            "func-- comment",
            "func/* comment */",
            "func#comment",
            # Union-based injections
            "func UNION SELECT",
            "func' UNION ALL SELECT",
            "func) UNION (SELECT",
            # Boolean-based injections
            "func OR 1=1",
            "func AND 1=1",
            "func' OR '1'='1",
            "func' AND '1'='1",
            # Time-based injections
            "func; WAITFOR DELAY",
            "func'||pg_sleep(10)||'",
            "func' AND SLEEP(5)",
            # Stacked queries
            "func; DROP TABLE",
            "func'; DELETE FROM",
            "func'; INSERT INTO",
            "func'; UPDATE SET",
            # Function calls
            "func(); SELECT",
            "func() AS",
            "func()\n        ",
            # Error-based injections
            "func' AND EXTRACTVALUE",
            "func' AND (SELECT COUNT",
            "func' AND UPDATEXML",
            # Blind injections
            "func' AND SUBSTRING",
            "func' AND ASCII",
            "func' AND LENGTH",
        ]

        for pattern in injection_patterns:
            assert not self.validate_function_name(pattern), (
                f"Injection pattern '{pattern}' should be blocked"
            )

    def test_unicode_and_encoding_attacks(self) -> None:
        """Test that validation handles Unicode and encoding-based attacks."""
        unicode_attacks = [
            "func\u0027DROP TABLE",  # Unicode single quote
            "func\u002dDROP TABLE",  # Unicode hyphen-minus
            "func\u003bDROP TABLE",  # Unicode semicolon
            "func\u0020DROP TABLE",  # Unicode space
            "func\u0009DROP TABLE",  # Unicode tab
            "func\u000aDROP TABLE",  # Unicode line feed
            "func\u000dDROP TABLE",  # Unicode carriage return
            "func\u00a0DROP TABLE",  # Unicode non-breaking space
        ]

        for attack in unicode_attacks:
            assert not self.validate_function_name(attack), (
                f"Unicode attack '{attack}' should be blocked"
            )


class TestSecurityBestPractices:
    """Test additional security best practices."""

    def test_whitelist_approach_demonstration(self) -> None:
        """Demonstrate a whitelist approach for maximum security."""
        # This would be the most secure approach
        allowed_functions = {
            "api.create_user",
            "api.update_user",
            "api.delete_user",
            "api.create_post",
            "api.update_post",
            "api.delete_post",
            "public.health_check",
            "auth.login",
            "auth.logout",
            "analytics.track_event",
        }

        def is_function_whitelisted(function_name):
            return function_name in allowed_functions

        # Test allowed functions
        assert is_function_whitelisted("api.create_user")
        assert is_function_whitelisted("public.health_check")
        assert is_function_whitelisted("auth.login")

        # Test disallowed functions (even if they look innocent)
        assert not is_function_whitelisted("api.create_admin")  # Not in whitelist
        assert not is_function_whitelisted("system.restart")  # Not in whitelist
        assert not is_function_whitelisted("debug.dump_data")  # Not in whitelist
        assert not is_function_whitelisted("'; DROP TABLE users; --")  # Malicious

        # Note: This approach requires maintaining a function registry
        # but provides the highest security level

    def test_function_signature_validation(self) -> None:
        """Test validation of function signatures to ensure they match expected patterns."""
        import re

        def validate_function_signature(function_name):
            # Pattern: schema_name.function_name where both parts are alphanumeric + underscores
            pattern = r"^[a-zA-Z][a-zA-Z0-9_]*\.[a-zA-Z][a-zA-Z0-9_]*$|^[a-zA-Z][a-zA-Z0-9_]*$"
            return bool(re.match(pattern, function_name))

        # Valid signatures
        valid_signatures = [
            "create_user",
            "api.create_user",
            "user_management.update_profile",
            "a.b",
            "test123.func456",
        ]

        for sig in valid_signatures:
            assert validate_function_signature(sig), f"'{sig}' should be a valid signature"

        # Invalid signatures
        invalid_signatures = [
            ".create_user",  # Can't start with dot
            "api.",  # Can't end with dot
            "api..create_user",  # Double dot
            "123.create_user",  # Can't start with number
            "api.123create",  # Function can't start with number
            "",  # Empty
            ".",  # Just dot
            "api.create-user",  # Hyphen not allowed
            "api create_user",  # Space not allowed
        ]

        for sig in invalid_signatures:
            assert not validate_function_signature(sig), f"'{sig}' should be an invalid signature"

==> fraiseql-1.6.0.tar.gz: fraiseql-1.6.0/tests/integration/auth/test_security_validation.py <==
"""Test security validation logic for FraiseQL."""

import pytest


@pytest.mark.security
class TestFunctionNameValidation:
    """Test the function name validation logic that prevents SQL injection."""

    def validate_function_name(self, function_name: str) -> bool:
        """Replicate the validation logic from db.py."""
        return function_name.replace("_", "").replace(".", "").isalnum()

    def test_valid_function_names(self) -> None:
        """Test that valid function names pass validation."""
        valid_names = [
            "create_user",
            "api.create_user",
            "schema123.function_name",
            "simple_function",
            "func123",
            "a",
            "a.b",
            "a_b",
            "func1_test2.schema3",
            "graphql.create_user",
            "public.health_check",
            "user_management.update_profile",
        ]

        for name in valid_names:
            assert self.validate_function_name(name), f"'{name}' should be valid"

    def test_malicious_function_names_rejected(self) -> None:
        """Test that malicious function names are rejected."""
        malicious_names = [
            "'; DROP TABLE users; --",
            "create_user'; DELETE FROM users; --",
            "func() UNION SELECT * FROM passwords",
            "func; INSERT INTO admin VALUES",
            "func'||pg_sleep(10)||'",
            'func"; TRUNCATE TABLE users; --',
            "func\nDROP DATABASE production",
            "func OR 1=1",
            "func) AS (SELECT password FROM users",
            "func/**/UNION/**/SELECT/**/*",
            "func' AND '1'='1",
            "func'; UPDATE users SET admin=true; --",
        ]

        for name in malicious_names:
            assert not self.validate_function_name(name), f"'{name}' should be rejected"

    def test_edge_case_function_names(self) -> None:
        """Test edge cases for function name validation."""
        edge_cases = [
            ("", False),  # Empty string
            ("func-name", False),  # Hyphen not allowed
            ("func name", False),  # Space not allowed
            ("func@domain", False),  # @ not allowed
            ("func$var", False),  # $ not allowed
            ("func#tag", False),  # # not allowed
            ("func%", False),  # % not allowed
            ("func*", False),  # * not allowed
            ("func()", False),  # Parentheses not allowed
            ("func[0]", False),  # Brackets not allowed
            ("func{}", False),  # Braces not allowed
            ("func<>", False),  # Angle brackets not allowed
            ("func|pipe", False),  # Pipe not allowed
            ("func\\backslash", False),  # Backslash not allowed
            ("func/slash", False),  # Forward slash not allowed
            ("func:colon", False),  # Colon not allowed
            ("func;semicolon", False),  # Semicolon not allowed
            ("func'quote", False),  # Single quote not allowed
            ('func"doublequote', False),  # Double quote not allowed
            ("func`backtick", False),  # Backtick not allowed
            ("func+plus", False),  # Plus not allowed
            ("func=equals", False),  # Equals not allowed
            ("func\ttab", False),  # Tab not allowed
            ("func\nnewline", False),  # Newline not allowed
            ("func\rcarriage", False),  # Carriage return not allowed
        ]

        for function_name, should_pass in edge_cases:
            result = self.validate_function_name(function_name)
            if should_pass:
                assert result, f"'{function_name}' should be valid but was rejected"
            else:
                assert not result, f"'{function_name}' should be invalid but was accepted"

    def test_validation_prevents_common_sql_injection_patterns(self) -> None:
        """Test that validation prevents common SQL injection attack patterns."""
        injection_patterns = [
            # Comment-based injections
            "func-- comment",
            "func/* comment */",
            "func#comment",
            # Union-based injections
            "func UNION SELECT",
            "func' UNION ALL SELECT",
            "func) UNION (SELECT",
            # Boolean-based injections
            "func OR 1=1",
            "func AND 1=1",
            "func' OR '1'='1",
            "func' AND '1'='1",
            # Time-based injections
            "func; WAITFOR DELAY",
            "func'||pg_sleep(10)||'",
            "func' AND SLEEP(5)",
            # Stacked queries
            "func; DROP TABLE",
            "func'; DELETE FROM",
            "func'; INSERT INTO",
            "func'; UPDATE SET",
            # Function calls
            "func(); SELECT",
            "func() AS",
            "func()\n        ",
            # Error-based injections
            "func' AND EXTRACTVALUE",
            "func' AND (SELECT COUNT",
            "func' AND UPDATEXML",
            # Blind injections
            "func' AND SUBSTRING",
            "func' AND ASCII",
            "func' AND LENGTH",
        ]

        for pattern in injection_patterns:
            assert not self.validate_function_name(pattern), (
                f"Injection pattern '{pattern}' should be blocked"
            )

    def test_unicode_and_encoding_attacks(self) -> None:
        """Test that validation handles Unicode and encoding-based attacks."""
        unicode_attacks = [
            "func\u0027DROP TABLE",  # Unicode single quote
            "func\u002dDROP TABLE",  # Unicode hyphen-minus
            "func\u003bDROP TABLE",  # Unicode semicolon
            "func\u0020DROP TABLE",  # Unicode space
            "func\u0009DROP TABLE",  # Unicode tab
            "func\u000aDROP TABLE",  # Unicode line feed
            "func\u000dDROP TABLE",  # Unicode carriage return
            "func\u00a0DROP TABLE",  # Unicode non-breaking space
        ]

        for attack in unicode_attacks:
            assert not self.validate_function_name(attack), (
                f"Unicode attack '{attack}' should be blocked"
            )


class TestSecurityBestPractices:
    """Test additional security best practices."""

    def test_whitelist_approach_demonstration(self) -> None:
        """Demonstrate a whitelist approach for maximum security."""
        # This would be the most secure approach
        allowed_functions = {
            "api.create_user",
            "api.update_user",
            "api.delete_user",
            "api.create_post",
            "api.update_post",
            "api.delete_post",
            "public.health_check",
            "auth.login",
            "auth.logout",
            "analytics.track_event",
        }

        def is_function_whitelisted(function_name) -> None:
            return function_name in allowed_functions

        # Test allowed functions
        assert is_function_whitelisted("api.create_user")
        assert is_function_whitelisted("public.health_check")
        assert is_function_whitelisted("auth.login")

        # Test disallowed functions (even if they look innocent)
        assert not is_function_whitelisted("api.create_admin")  # Not in whitelist
        assert not is_function_whitelisted("system.restart")  # Not in whitelist
        assert not is_function_whitelisted("debug.dump_data")  # Not in whitelist
        assert not is_function_whitelisted("'; DROP TABLE users; --")  # Malicious

        # Note: This approach requires maintaining a function registry
        # but provides the highest security level

    def test_function_signature_validation(self) -> None:
        """Test validation of function signatures to ensure they match expected patterns."""
        import re

        def validate_function_signature(function_name) -> None:
            # Pattern: schema_name.function_name where both parts are alphanumeric + underscores
            pattern = r"^[a-zA-Z][a-zA-Z0-9_]*\.[a-zA-Z][a-zA-Z0-9_]*$|^[a-zA-Z][a-zA-Z0-9_]*$"
            return bool(re.match(pattern, function_name))

        # Valid signatures
        valid_signatures = [
            "create_user",
            "api.create_user",
            "user_management.update_profile",
            "a.b",
            "test123.func456",
        ]

        for sig in valid_signatures:
            assert validate_function_signature(sig), f"'{sig}' should be a valid signature"

        # Invalid signatures
        invalid_signatures = [
            ".create_user",  # Can't start with dot
            "api.",  # Can't end with dot
            "api..create_user",  # Double dot
            "123.create_user",  # Can't start with number
            "api.123create",  # Function can't start with number
            "",  # Empty
            ".",  # Just dot
            "api.create-user",  # Hyphen not allowed
            "api create_user",  # Space not allowed
        ]

        for sig in invalid_signatures:
            assert not validate_function_signature(sig), f"'{sig}' should be an invalid signature"
